
# q-expansion of newform 9450.2.a.i, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 9450
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [0], [-1], [-3], [5], [-4], [1], [-8], [-10], [-8], [-4], [-9], [1], [13], [3], [14], [6], [-11], [0], [7], [-10], [15], [11], [18], [3], [14], [10], [5], [-17], [17], [-4], [-7], [4], [-12], [-6], [-18], [12], [-12], [-6], [-5], [-22], [-5], [10], [5], [-3], [6], [26], [-9], [-2], [11], [-4], [6], [8], [12], [12], [-10], [15], [8], [30], [4], [6], [10], [18], [25], [-13], [20], [-22], [-12], [-24], [-6], [13], [2], [10], [-14], [24], [-18], [14], [6], [-26], [26], [1], [11], [5], [-1], [-12], [-22], [-38], [7], [11], [-4], [6], [-7], [36], [40], [35], [26], [27], [-4], [35], [8], [-2], [19], [18], [-34], [13], [-28], [6], [-3], [-28], [4], [36], [22], [7], [-30], [-14], [-10], [-3], [-30], [-12], [-32], [4], [-4], [20], [25], [-40], [27], [14], [2], [-11], [-44], [-18], [10], [-24], [-14], [-26], [42], [18], [8], [28], [21], [6], [-33], [-26], [-2], [26], [-26], [52], [-15], [-22], [14], [30], [-52], [21], [37], [-43], [14], [-15], [47], [-7], [-42], [-42], [-29], [42], [-33], [-16], [52], [14], [46], [32], [39], [42], [16], [62], [-35], [-3], [-10], [4], [24], [8], [29], [-46], [-22], [36], [-50], [-39], [12], [-28], [7], [-36], [34], [-60], [34], [21], [16], [-40], [-58], [-12], [28], [7], [46], [67], [-45], [0], [18], [-56], [-15], [26], [-60], [-13], [-4], [19], [45], [12], [22], [2], [-12], [-42], [6], [29], [-17], [-31], [6], [15], [-11], [-6], [36], [35], [-16], [-3], [-50], [27], [-28], [-24], [26], [-21], [-20], [45], [9], [-6], [-30], [-20], [4], [27], [-30], [24], [14], [54], [-42], [-12], [10], [78], [-40], [56], [-48], [76], [74], [-2], [12], [-74], [8], [-36], [28], [-77], [-40], [-53], [-6], [-50], [48], [33], [4], [-33], [-58], [74], [9], [-36], [12], [-41], [17], [-78], [12], [-20], [8], [-16], [4], [0], [-26], [18], [41], [69], [-18], [27], [75], [38], [40], [-5], [-56], [74], [-12], [-2], [67], [37], [48], [24], [76], [32], [80], [38], [21], [-46], [77], [-6], [-66], [-81], [12], [-58], [30], [40], [-7], [36], [64], [-46], [65], [25], [-31], [20], [-32], [-40], [-30], [-14], [85], [93], [-32], [-21], [40], [40], [-45], [40], [-2], [-2], [-76], [71], [-39], [58], [6], [31], [-64], [-15], [78], [-38], [-27], [-16], [-4], [90], [-70], [-22], [-68], [-6], [-93], [-66], [-36], [21], [-40], [68], [-64], [18], [2], [-68], [-63], [39], [17], [66], [36], [-11], [74], [-64], [15], [-39], [-74], [95], [-57], [71], [-57], [57], [-85], [-21], [-32], [13], [-29], [-48], [-85], [24], [76], [-22], [-99], [-60], [-56], [25], [2], [-58], [30], [31], [-27], [60], [57], [-55], [48], [-30], [76], [60], [-9], [15], [70], [18], [-26], [-99], [-45], [64], [30], [69], [-96], [-11], [-8], [56], [-6], [66], [20], [18], [0], [70], [-16], [-44], [-20], [-81], [95], [63], [-47], [94], [92], [16], [53], [-36], [11], [-1], [82], [-6], [-30], [-41], [-88], [-32], [-11], [-16], [-14], [-21], [16], [16], [-95], [-12], [-6], [3], [-96], [-50], [-55], [37], [22], [-36], [44], [109], [-66], [31], [88], [-54], [24], [43], [-3], [-93], [-104], [5], [92], [75], [11], [55], [-25], [-53], [-36], [60], [7], [78], [63], [60], [30], [96], [97], [34], [44], [40], [94], [29], [-115], [-38], [54], [74], [6], [-13], [54], [42], [-92], [1], [10], [26], [-8], [48], [5], [-79], [30], [-22], [70], [-20], [19], [14], [98], [94], [10], [-72], [-22], [-78], [-89], [0], [70], [-77], [48], [63], [101], [27], [-27], [-17], [77], [34], [-47], [-7], [40], [6], [-52], [12], [-102], [-30], [103], [63], [-50], [-117], [-20], [54], [-15], [-9], [55], [-94], [33], [-63], [-84], [-46], [6], [-74], [-26], [-101], [-112], [-4], [48], [-36], [52], [-67], [62], [44], [-54], [31], [-17], [-69], [-20], [51], [18], [107], [65], [12], [26], [82], [32], [-35], [-2], [-101], [-111], [86]]
